// -*- mode:C++; compile-command: "g++ -g mps.cc -lgiac -lmpfi -lmpfr -lgmp && g++ -g mps.cc -DHAVE_LIBMPS -lgiac -lmpfi -lmpfr -lgmp -lmps" -*-
// Polynomial roots for small/medium degrees 
#include <fstream>
#include <iostream>
#include <vector>
#include <complex>
#include <algorithm>
#include <iomanip>
#include <cmath>
#define MPFI_CERT
//#define HAVE_LIBMPS 1
#ifdef HAVE_LIBMPS
#include <mps/mps.h>
#include <stdlib.h>
#include <stdio.h>
#include <gmp.h>
#include <string.h>
#endif

int Nmax=25; // max number of iter, change with export GIAC_NMAX=50
//int ABERTH_NBITSMAX=8192; // default max multi-precision, change with e.g. export GIAC_N1=200
int nbits_start=128; // GIAC_N0
double cluster_step=1e-4; // if step>cluster_step, no cluster analysis, export GIAC_STEP=1e-3
double cluster_dp=0.2; // if step*sum_{j!=i}(inv(approx_root[i]-approx_root[j])>cluster_dp, then do cluster analysis, export GIAC_DP=.1
//int debug_infolevel=0; // export GIAC_DEBUG=1 or 2
double MINREAL=-1e307;

#include <giac/config.h>
#include <giac/giac.h>
using namespace giac;
using namespace std;
context * contextptr=0;

// single precision
#if defined __x86_64__ || defined __i386__
#define LDBL80 // 80 bit long double
#endif

#ifdef LDBL80
typedef complex<long double> fdbl;
#else
typedef complex<double> fdbl;
//typedef complex<float> fdbl;
#endif
inline double absdbl(const fdbl & x){
  return abs(x);
}
inline fdbl re(const fdbl & x){
  return fdbl(x.real(),0);
}
inline double redbl(const fdbl & x){
  return x.real();
}
inline double imdbl(const fdbl & x){
  return x.imag();
}
inline bool is_exactly_zero(const fdbl & x){
  return x.real()==0 && x.imag()==0;
}
fdbl inv(const fdbl z){
  return fdbl(1.0)/z;
}

bool fdbl_less(const fdbl & x,const fdbl & y){
  if (x.real()!=y.real())
    return x.real()<y.real();
  return x.imag()<y.imag();
}



typedef vector<fdbl> vfdbl;
ostream & operator << (ostream & os,const vfdbl & P){
  os << "[";
  for (int i=0;i<P.size();++i){
    os << P[i].real() << "+" << P[i].imag() << "*i";
    if (i!=P.size()-1)
      os << ",";
  }
  os << "]";
  return os;
}

void print(const vfdbl & P){
  cout << P << "\n";
}

fdbl horner_rem(const vfdbl & P,fdbl x,vfdbl * Q){
  if (Q)
    Q->clear(); 
  if (P.empty())
    return 0.0;
  size_t s=P.size();
  if (Q)
    Q->reserve(s-1);
  fdbl r=0;
  if (Q){
    for (size_t i=0;;){
      r=r*x+P[i];
      ++i;
      if (i==s)
	break;
      Q->push_back(r);
    }
  }
  else {
    for (size_t i=0;i<s;++i){
      r=r*x+P[i];
    }
  }
  return r;
}

double l1(const vfdbl & P,const fdbl & z){
  double r=absdbl(z),rn=1,res=0;
  for (size_t i=1;i<=P.size();++i){
    double anrn=absdbl(P[P.size()-i])*rn;
    rn *= r;
    if (anrn<=res)
      continue;
    res=anrn;
  }
  return res;
}

double l1(const vfdbl & P){
  double res=0;
  for (size_t i=1;i<=P.size();++i){
    double an=absdbl(P[P.size()-i]);
    if (an<=res)
      continue;
    res=an;
  }
  return res;
}

// shift roots such that sum of roots=0
fdbl find_shift(const vfdbl & R){
  int n=R.size()-1;
  if (n==0)
    return 0;
  fdbl d=R[1]/fdbl(n)/R[0];
  return fdbl(floor(d.real()),floor(d.imag()));
}

vfdbl shift(const vfdbl & P0,fdbl & d,bool l1chk=true){
  vfdbl P(P0),Q,R;
  double D=l1(P0);
  if (isinf(D)){
    d=0;
    return P0;
  }
  int n=P.size();
  for (int i=0;i<n;++i){
    fdbl Pz=horner_rem(P,d,&Q);
    if (l1chk && absdbl(Pz)>D){
      d=0;
      return P0; // no shift, it would increase the l1 norm
    }
    R.push_back(Pz);
    P.swap(Q);
  }
  reverse(R.begin(),R.end());
  return R;
}

fdbl sum(const vfdbl & P){
  fdbl r(0.0);
  for (size_t i=0;i<P.size();++i)
    r += P[i];
  return r;
}

double sumabs(const vfdbl & P){
  double r(0.0);
  for (size_t i=0;i<P.size();++i)
    r += absdbl(P[i]);
  return r;
}

bool linreg(const vector<double> & x,const vector<double> & y,double & a,double &b,double & r){
  size_t n=x.size();
  if (n!=y.size())
    return false;
  double X=0,Y=0,XY=0,X2=0,Y2=0;
  for (size_t i=0;i<n;++i){
    X += x[i];
    Y += y[i];
  }
  X /= n; Y /= n;
  for (size_t i=0;i<n;++i){
    double xx=x[i]-X,yy=y[i]-Y;
    XY += xx*yy;
    X2 += xx*xx;
    Y2 += yy*yy;
  }
  if (X2==0)
    return false;
  XY /= n; X2 /= n; Y2 /= n;
  a=XY/X2;
  b=Y-a*X;
  r=Y2==0?1:XY/sqrt(X2)/sqrt(Y2);
  return true;
}

double find_scale(const vfdbl & P){
  int n=P.size()-1;
  vector<double> x,y;
  for (int i=0;i<=n;++i){
    double a=absdbl(P[i]);
    if (a==0)
      continue;
    if (isinf(a))
      return 1;
    x.push_back(n-i);
    y.push_back(log(a));
  }
  if (x.empty())
    return 1;
  double a,b,r;
  linreg(x,y,a,b,r);
  return exp(-a);
}

void rescale(vfdbl & P, fdbl l){
  if (l==fdbl(1)) return;
  fdbl ll=l;
  for (int i=P.size()-2;i>=0;--i){
    P[i] = ll*P[i];
    ll = ll*l;
  }
  fdbl c=P[0];
  for (size_t i=0;i<P.size();++i)
    P[i] = P[i]/c;
}


bool dkw(const vfdbl & P0,int N,double eps,vfdbl & R){
  bool real=true;
  for (size_t i=0;real && i<P0.size();++i){
    if (imdbl(P0[i])!=0)
      real=false;
  }
  fdbl dr=-find_shift(P0);
  vfdbl P=shift(P0,dr);
  double l=find_scale(P);
  rescale(P,l);
  int deg=P0.size()-1;
  // init R if not already done
  if (R.size()!=deg){
    fdbl w(0.0,2.0*M_PI/deg);
    w=exp(w);
    R.push_back(fdbl(1.0));
    fdbl wk=w;
    for (int i=1;i<deg;++i){
      R.push_back(wk);
      wk = wk*w;
    }
  }
  vfdbl newR(R);
  int ok=0;
  N *= P.size();
  double delta=0;
  for (int k=0;k<N;++k){
    delta=0;
    for (int i=0;i<deg;++i){
      fdbl & zi=R[i];
      fdbl d=horner_rem(P,zi,0);
      fdbl p(1.0);
      for (int j=0;j<deg;++j){
	if (j==i) continue;
	p = p*(zi-R[j]);
      }
      if (absdbl(p)==0)
	return false;
      d = d/p;
      delta += absdbl(d);
      newR[i]=zi-d;
    }
    newR.swap(R);
    if (debug_infolevel>0)
      *logptr(contextptr) << k << ": " << delta << "\n";
    if (delta<eps*sumabs(R))
      ++ok;
    else
      ok=0;
    if (ok==2){
      // rescale and translate
      for (size_t j=0;j<R.size();++j){
        R[j] = fdbl(l)*R[j]+dr;
      }
      sort(R.begin(),R.end(),fdbl_less);
      return true;
    }
  }
  *logptr(contextptr) << "Durand-Kerner-Weierstrass: too many iterations " << N << " delta " << delta << "\n";
  return false;
}

struct int_2double {
  int i;
  double theta,norm;
};

struct int_double {
  int i;
  double d;
};

bool norm_sort(const int_double & a,const int_double & b){
  if (a.d!=b.d)
    return a.d<b.d;
  return a.i<b.i;
}

double norm(double dx,double dy){
  if (dx==0 && dy==0)
    return 0;
  if (dx<0) dx=-dx;
  if (dy<0) dy=-dy;
  if (dx>dy){
    double z=dy/dx;
    return dx*sqrt(1+z*z);
  }
  else {
    double z=dx/dy;
    return dy*sqrt(1+z*z);
  }
}

bool graham_sort_function(const int_2double & a,const int_2double & b){
  if (a.theta==b.theta)
    return b.norm>a.norm;
  return b.theta>a.theta;
}

double cross_prod(const vfdbl & v,int a,int b,int c){
  fdbl ab=v[b]-v[a],ac=v[c]-v[a];
  double A=redbl(ab),B=imdbl(ab),C=redbl(ac),D=imdbl(ac);
  return A*D-B*C;
}

vector<int> convexhull(const vfdbl & v){
  int s=v.size(),imin=0;
  if (s==1)
    return vector<int>(1,0);
  // find origin
  double ymin=imdbl(v[0]),ycur,xmin=redbl(v[0]),xcur;
  for (int i=1;i<s;++i){
    ycur=imdbl(v[i]);xcur=redbl(v[i]);
    if (ymin>ycur || (ymin==ycur && xmin>xcur) ){
      imin=i; ymin=ycur; xmin=xcur;
    }
  }
  vector<int_2double> ls;
  for (int j=0;j<s;++j){
    if (j!=imin){
      double dx=redbl(v[j])-xmin,dy=imdbl(v[j])-ymin;
      int_2double s={j,atan2(dy,dx),norm(dx,dy)};
      ls.push_back(s);
    }
  }
  sort(ls.begin(),ls.end(),graham_sort_function);
  vector<int> res; res.push_back(imin); res.push_back(ls[0].i);
  int ress=2;
  for (int j=1;j<s-1;++j){
    int icur=ls[j].i;
    double o=cross_prod(v,res[ress-2],res[ress-1],icur);
    if (o==0)
      res[ress-1]=icur;
    else {
      if (o>0){
        res.push_back(icur);
        ++ress;
      }
      else {
        while (ress>2 && o<0){
          res.pop_back();
          ress--;
          o=cross_prod(v,res[ress-2],res[ress-1],icur);
        }
        res.push_back(icur);
        ++ress;
      }
    }
  }
  return res;
}

double init_R(const vfdbl & P,vfdbl & R){
  R.clear();
  int n=P.size()-1;
  vfdbl l;
  vector<int> lpos;
  double M=0;
  for (int i=0;i<=n;++i){
    long double ai=abs(P[n-i]);
    if (ai==0)
      continue;
    l.push_back(fdbl(double(i),log(ai)));
    lpos.push_back(i);
  }
  vector<int> pos=convexhull(l);
  // find real positions (since coeffs==0 were removed)
  for (int i=0;i<pos.size();++i){
    pos[i]=lpos[pos[i]];
  }
  sort(pos.begin(),pos.end());
  if (pos[0]!=0)
    pos.insert(pos.begin(),0);
  if (pos.back()!=n)
    pos.push_back(n);
  int count=0;
  for (int i=1;i<pos.size();++i){
    int dk=pos[i]-pos[i-1];
    double uk=pow(abs(P[n-pos[i]]/P[n-pos[i-1]]),-1.0/dk);
    if (uk==0) return 0;
    if (uk>M)
      M=uk;
    double sigma=0.7;
    for (int j=0;j<dk;++j){
      double theta=sigma+2*M_PI*i/n+2*M_PI*j/dk;
      fdbl z(uk*cos(theta),uk*sin(theta));
      R.push_back(z);
    }
  }
  return M;
}

// find r=P(x) and r1=diff(P)(x)
// if |x|>1 this may overflow
void horner2(const vfdbl & P,fdbl x,fdbl & r,fdbl & r1){
  r=r1=0;
  if (P.empty())
    return ;
  size_t s=P.size()-1;
  for (size_t i=0;i<s;++i){
    r=r*x+P[i];
    r1=r1*x+r;
  }
  r=r*x+P[s];
}

double l1norm(const vfdbl & v){
  double r=0;
  for (int i=0;i<v.size();++i)
    r += absdbl(v[i]);
  return r;
}

void div(vfdbl & v,double r){
  for (int i=0;i<v.size();++i)
    v[i] /= r;
}

// approx largest root in norm, using a few iterations of power method
double largest(const vfdbl & fP,int maxiter=100,double eps=1e-6){
  int deg=fP.size()-1;
  if (debug_infolevel)
    *logptr(contextptr) << "power method " << fP << "\n";
  vfdbl v(deg);
  for (int i=0;i<deg;++i)
    v[i]=fdbl(rand()/(1.0+RAND_MAX),rand()/(1.0+RAND_MAX));
  div(v,l1norm(v));
  vfdbl w(deg);
  double oldR(0.0);
  for (int j=0;j<maxiter;++j){
    for (int i=0;i<deg-1;++i){
      w[i+1]=v[i];
    }
    fdbl r(0,0);
    for (int i=0;i<deg;++i){
      r += fP[i+1]*v[i];
    }
    w[0]= -r/fP[0];
    double R=l1norm(w);
    if (debug_infolevel)
      *logptr(contextptr) << "power method " << j << " " << R << "\n";
    div(w,R);
    if (j>5 && abs(1-R/oldR)<eps)
      return R;
    v.swap(w);
    oldR=R;
  }
  return oldR;
}

#define REV 1

void translate_shift(vfdbl & R,const fdbl & l,const fdbl & dr){
  if (l==fdbl(1) && dr==fdbl(0))
    return;
  // rescale and translate
  for (size_t j=0;j<R.size();++j){
    R[j] = l*R[j] + dr;
  }
}

bool aberth_singleprec(const vfdbl & P0,int N,double eps,vfdbl & R,int cluster_start,int cluster_afterend,vector<short int> & zi_done,int afteriter,GIAC_CONTEXT){
  int deg=P0.size()-1;
  bool doing_cluster=cluster_start>0 || cluster_afterend<deg;
  vfdbl P(P0);
  // init R if not already done
  fdbl dr(0);
  fdbl l(1);
  if (R.size()!=deg){
    dr=-find_shift(P);
    P=shift(P,dr);
    l=1;//find_scale(P);
    rescale(P,l);
    double res=init_R(P,R);
    if (res==0 || isinf(res)){
      R.clear();
      return false;
    }
  }
  vfdbl Prev(P);
  reverse(Prev.begin(),Prev.end());
  vfdbl newR(deg);
  int ok=0; double delta;
  for (int k=0;k<N;++k){
    delta=0;
    if (doing_cluster){
      int K=cluster_start;
      for (;K<cluster_afterend;++K){
        if (!zi_done[K])
          break;
      }
      if (K==cluster_afterend)
        ++ok;
    }
    for (int i=cluster_start;i<cluster_afterend;++i){
      if (!ok && zi_done[i]){ // always do the last iteration for accuracy
        newR[i]=R[i];
        continue;
      }
      fdbl & zi=R[i];
      fdbl d,d1;
      if (absdbl(zi)>1){
        fdbl gamma(inv(zi));
        horner2(Prev,gamma,d,d1);
        if (!is_exactly_zero(d)){
          d=gamma*(fdbl(deg)-gamma*d1/d);
          if (is_exactly_zero(d)){
            translate_shift(R,l,dr);
            if (debug_infolevel)
              *logptr(contextptr) << "Aberth_single precision |z|>1 trying to invert 0\n";
            return false;
          }
          d=inv(d);
        }
      }
      else {
        horner2(P,zi,d,d1);
        if (is_exactly_zero(d1)){
          translate_shift(R,l,dr);
          if (debug_infolevel)
            *logptr(contextptr) << "Aberth_single precision |z|<=1 trying to invert 0\n";
          return false;
        }
        d=d/d1;
      }
      fdbl p(0); bool binv=true;
      for (int j=0;j<deg;++j){
	if (j==i) continue;
        fdbl dz=(zi-R[j]);
        if (is_exactly_zero(dz)){
          binv=false;
          break; // return false;
        }
	p += inv(dz);
      }
      double dd=absdbl(d),pp=absdbl(p);
      double abszi=absdbl(zi);
      if (debug_infolevel>2)
        *logptr(contextptr) << "cluster? i=" << i << ", delta=" << dd << ", cluster_step=" << cluster_step << ", p=" << pp << ", d*p=" << dd*pp << " " << cluster_dp << "\n";
      if (!ok
          && k>8
          && !doing_cluster && dd<=cluster_step*abszi && dd*pp>=cluster_dp){
        // if (ok) continue;
        // cluster of roots, find all roots in this cluster
        int cend=i+1;
        fdbl sumR=zi;
        for (int k=cend;k<deg;++k){
          if (abs(zi-R[k])<2*deg/pp){
            swap(R[cend],R[k]);
            swap(zi_done[cend],zi_done[k]);
            sumR+=R[cend];
            ++cend;
          }
        }
        if (cend-i>1){
          int Nc=cend-i;
          // cluster is from i to cend-1 included
          fdbl z=sumR/fdbl(Nc); // center of gravity of cluster
          vfdbl Pdiff;
          --Nc;
          for (int l=0;l<=deg-Nc;++l){
            fdbl h=P[l];
#ifdef LDBL80
            for (int k=deg-l;k>deg-l-Nc;--k){
              h=((long double) k)*h;
            }
#else
            for (int k=deg-l;k>deg-l-Nc;--k){
              h=((double) k)*h;
            }
#endif
            Pdiff.push_back(h);
          }
          fdbl z1,z2;
          horner2(Pdiff,z,z1,z2);
          if (!is_exactly_zero(z2)){
            fdbl delta=z1/z2;
            z=z-delta;
            for (;0;){ // disabled
              horner2(P,z,z1,z2);
              if (!is_exactly_zero(z1))
                break;
              z += fdbl(1-1e-17)*z;
            }
          }
          // shift P and reverse (roots become inverse of roots)
          vfdbl Pcluster(shift(P,z)),Rcluster(deg),initR(deg);
          reverse(Pcluster.begin(),Pcluster.end());
          if (is_exactly_zero(Pcluster[0])){
            Pcluster[0]=z/pow(fdbl(2),64);
          }
          init_R(Pcluster,initR);
          int count=deg-1;
          for (int k=0;k<deg;++k){
            if (k>=i && k<cend){
              Rcluster[k]=initR[count];
              --count;
              //dbl a(0.0,0.7+(k-i)*2*M_PI/(cend-i));
              //Rcluster[k]=D*exp(a);
            }
            else
              Rcluster[k]=inv(R[k]-z);
          }
          vector<short int> old(zi_done);        
          if (debug_infolevel)
            *logptr(contextptr) << "aberth single cluster=" << i << "," << cend << "\n";
          // recursive call of aberth
          bool b=aberth_singleprec(Pcluster,N,eps,Rcluster,i,cend,zi_done, 2/* afteriter*/,contextptr);
          for (int k=i;k<cend;++k){
            R[k]=newR[k]=inv(Rcluster[k])+z;
          }
          if (b)
            continue;
          translate_shift(R,l,dr);
          if (debug_infolevel)
            *logptr(contextptr) << "Aberth_single precision cluster split failure\n";
          return false;
        }
      }
      double absdz=abszi==0?absdbl(d):absdbl(d)/abszi;
      if (binv)
        d=d/(fdbl(1)-d*p);
      else
        d=d/(fdbl(1-rand()/2.0/RAND_MAX));
      if (absdz<eps/deg){
        if (debug_infolevel>1 && !zi_done[i])
          *logptr(contextptr) << "New root found " << zi-d << "\n";
        zi_done[i]=1;
      }
      delta += absdz;
      newR[i]=zi-d;
      R[i]=newR[i]; // comment to avoid immediate update
    }
    newR.swap(R);
    int count=0;
    for (int k=0;k<deg;k++){
      if (zi_done[k])
        ++count;
    }
    if (debug_infolevel>0){
      *logptr(contextptr) << "Aberth single prec iter " << k <<  " found=" << count << " cluster_search_start=" << cluster_start << ".." << cluster_afterend << " delta=" << delta << " time " << clock()*1e-6 << "\n";
      if (debug_infolevel>2){
        *logptr(contextptr) << "Current approx roots " << R << "\n";
      }
    }
    if (doing_cluster){
      if (ok>=afteriter)
        return true;
      continue;
    }
    if (//count==deg
        delta<=eps*(ok?deg:deg-count) || count==deg
        )
      ++ok;
    else
      ok=0;
    if (ok>=afteriter){
      translate_shift(R,l,dr);
      sort(R.begin(),R.end(),fdbl_less);
      return true;
    }
  }
  if (debug_infolevel)
    *logptr(contextptr) << "Aberth single precision: too many iterations " << N << " delta " << delta << "\n";
  return false;  
}

bool read_poly(const string & filename,vfdbl & P){
  P.clear();
  ifstream i(filename.c_str());
  while (1){
    fdbl z;
    i >> z;
    if (i.eof())
      break;
    P.push_back(z);
  }
  return true;
}

vfdbl p_coeff(const vfdbl & R){
  vfdbl P;
  P.reserve(R.size()+1);
  P.push_back(fdbl(1));
  for (size_t i=0;i<R.size();++i){
    fdbl z=R[i];
    // P*(x-z)
    P.push_back(-z*P.back());
    for (size_t j=P.size()-2;j>=1;--j){
      P[j] -= z*P[j-1];
    }
  }
  return P;
}

// max distance between 2 elements of R
double ecart(const vfdbl & R){
  int n=R.size();
  double d=1e307;
  for (int i=0;i<n;++i){
    const fdbl & Ri=R[i];
    for (int j=0;j<n;++j){
      if (j==i) continue;
      double dd=absdbl(Ri-R[j]);
      if (dd<d)
        d=dd;
    }
  }
  return d;
}

// multi-precision
typedef gen dbl;
inline double absdbl(const dbl & x){
  dbl g=abs(evalf_double(x,1,contextptr));
  return g.type==_DOUBLE_?g._DOUBLE_val:0;
}
inline double logabsdbl(const dbl & x){
  if (is_exactly_zero(x))
    return MINREAL; // -707;
  dbl g=x;
  if (g.type!=_REAL)
    g=accurate_evalf(g,128);
  g=ln(abs(g,contextptr),contextptr);
  g=evalf_double(g,1,contextptr);
  return g.type==_DOUBLE_?g._DOUBLE_val:0;
}
inline double redbl(const dbl & x){
  return evalf_double(re(x,contextptr),1,contextptr)._DOUBLE_val;
}
inline double imdbl(const dbl & x){
  return evalf_double(im(x,contextptr),1,contextptr)._DOUBLE_val;
}
inline dbl conj(const dbl & x){
  return conj(x,contextptr);
}
inline dbl re(const dbl & x){
  return re(x,contextptr);
}
inline dbl im(const dbl & x){
  return im(x,contextptr);
}
inline dbl exp(const dbl & x){
  return exp(x,contextptr);
}
inline dbl inv(const dbl & x){
  return inv(x,contextptr);
}
bool dbl_less(const dbl & x,const dbl & y){
  gen xr,xi,yr,yi;
  reim(x,xr,xi,contextptr);
  reim(y,yr,yi,contextptr);
  if (xi==0 && yi!=0)
    return true;
  if (xi!=0 && yi==0)
    return false;
  if (xr!=yr)
    return is_strictly_greater(yr,xr,contextptr);
  return is_strictly_greater(yi,xi,contextptr);
  //return !islesscomplexthanf(x,y);
}

bool dblrayon_less(const dbl & x,const dbl & y){
  return dbl_less(x[0],y[0]);
}

typedef vecteur vdbl;
#if 0
ostream & operator << (ostream & os,const vdbl & P){
  for (int i=0;i<P.size();++i)
    os << P[i] << " ";
  return os;
}
#endif

void accurate_evalf(vdbl & P,int nbits){
  for (int i=0;i<P.size();++i)
    P[i]=accurate_evalf(P[i],nbits);
}

dbl horner_rem(const vdbl & P,dbl x,vdbl * Q){
  if (Q)
    Q->clear(); 
  if (P.empty())
    return 0.0;
  size_t s=P.size();
  if (Q)
    Q->reserve(s-1);
  dbl r=0;
  if (Q){
    for (size_t i=0;;){
      r=r*x+P[i];
      ++i;
      if (i==s)
	break;
      Q->push_back(r);
    }
  }
  else {
    for (size_t i=0;i<s;++i){
      r=r*x+P[i];
    }
  }
  return r;
}

double loglinf(const vdbl & P,const dbl & z){
  double r=logabsdbl(z),rn=1,res=MINREAL;
  for (size_t i=1;i<=P.size();++i){
    double anrn=logabsdbl(P[P.size()-i])*rn;
    rn *= r;
    if (anrn<=res)
      continue;
    res=anrn;
  }
  return res;
}

double loglinf(const vdbl & P){
  double res=MINREAL;
  for (size_t i=1;i<=P.size();++i){
    double an=logabsdbl(P[P.size()-i]);
    if (an<=res)
      continue;
    res=an;
  }
  return res;
}

// shift roots such that sum of roots=0
dbl find_shift(const vdbl & R,GIAC_CONTEXT){
  int n=R.size()-1;
  if (n==0)
    return 0;
  return _floor(R[1]/dbl(n)/R[0],contextptr);
}

vdbl shift(const vdbl & P0,dbl & d,bool l1chk=true){
  double D=loglinf(P0);
#if 0
  dbl chk=horner_rem(P0,d,0);
  if (l1chk && logabsdbl(chk)>D){
    d=0;
    return P0;
  }
  vdbl res=taylor(P0,d);
  for (int i=0;i<res.size();++i){
    if (l1chk && logabsdbl(res[i])>D){
      d=0;
      return P0; // no shift, it would increase the linf norm of the polynomial
    }
  }
  return res;
#endif
  vdbl P(P0),Q,R;
  int n=P.size();
  for (int i=0;i<n;++i){
    dbl Pz=horner_rem(P,d,&Q);
    if (l1chk && logabsdbl(Pz)>D){
      d=0;
      return P0; // no shift, it would increase the linf norm of the polynomial
    }
    R.push_back(Pz);
    P.swap(Q);
  }
  reverse(R.begin(),R.end());
  return R;
}

dbl sum(const vdbl & P){
  dbl r(0.0);
  for (size_t i=0;i<P.size();++i)
    r += P[i];
  return r;
}

double sumabs(const vdbl & P){
  double r(0.0);
  for (size_t i=0;i<P.size();++i)
    r += absdbl(P[i]);
  return r;
}

double find_scale(const vdbl & P){
  int n=P.size()-1;
  vector<double> x,y;
  for (int i=0;i<=n;++i){
    double a=logabsdbl(P[i]);
    if (a==MINREAL)
      continue;
    x.push_back(n-i);
    y.push_back(a);
  }
  if (x.empty())
    return 1;
  double a,b,r;
  linreg(x,y,a,b,r);
  return exp(-a);
}

void rescale(vdbl & P, dbl l){
  if (l==dbl(1)) return;
  dbl ll=l;
  for (int i=P.size()-2;i>=0;--i){
    P[i] = ll*P[i];
    ll = ll*l;
  }
  dbl c=P[0];
  for (size_t i=0;i<P.size();++i)
    P[i] = P[i]/c;
}

bool dkw(const vdbl & P0,int N,double eps,vdbl & R){
  bool real=true;
  for (size_t i=0;real && i<P0.size();++i){
    if (imdbl(P0[i])!=0)
      real=false;
  }
  dbl dr=-find_shift(P0,contextptr);
  vdbl P=shift(P0,dr);
  double l=find_scale(P);
  rescale(P,l);
  int deg=P0.size()-1;
  // init R if not already done
  if (R.size()!=deg){
    dbl w(0.0,2.0*M_PI/deg);
    w=exp(w);
    R.push_back(dbl(1.0));
    dbl wk=w;
    for (int i=1;i<deg;++i){
      R.push_back(wk);
      wk = wk*w;
    }
  }
  vdbl newR(R);
  int ok=0;
  N *= P.size();
  double delta=0;
  for (int k=0;k<N;++k){
    delta=0;
    for (int i=0;i<deg;++i){
      dbl & zi=R[i];
      dbl d=horner_rem(P,zi,0);
      dbl p(1.0);
      for (int j=0;j<deg;++j){
	if (j==i) continue;
	p = p*(zi-R[j]);
      }
      if (is_exactly_zero(p)==0)
	return false;
      d = d/p;
      delta += absdbl(d);
      newR[i]=zi-d;
    }
    newR.swap(R);
    if (debug_infolevel>0)
      *logptr(contextptr) << k << ": " << delta << "\n";
    if (delta<eps*sumabs(R))
      ++ok;
    else
      ok=0;
    if (ok==2){
      // rescale and translate
      for (size_t j=0;j<R.size();++j){
        R[j] = dbl(l)*R[j]+dr;
      }
      sort(R.begin(),R.end(),dbl_less);
      return true;
    }
  }
  *logptr(contextptr) << "Durand-Kerner-Weierstrass: too many iterations " << N << " delta " << delta << "\n";
  return false;
}

// find r=P(x) and r1=diff(P)(x)
// if |x|>1 this may overflow
void horner2(const vdbl & P,dbl x,dbl & r,dbl & r1){
  r=r1=0;
  if (P.empty())
    return ;
  size_t s=P.size()-1;
  for (size_t i=0;i<s;++i){
    r=r*x+P[i];
    r1=r1*x+r;
  }
  r=r*x+P[s];
}

#ifdef HAVE_LIBMPFR
// computes an estimate of the number of bits of precision lost
void add(mpfr_t & rr,mpfr_t & ri,const gen & Pi,int nbits,long & loss,long & size){
  long rr0,rr1,delta=0;
  mpfr_get_d_2exp(&rr0,rr,MPFR_RNDN);
  if (rr0>size)
    size=rr0;
  if (Pi.type==_REAL){
    mpfr_t & inf=Pi._REALptr->inf;
    if (mpfr_zero_p(inf))
      return ;
    mpfr_get_d_2exp(&rr1,inf,MPFR_RNDN);
    if (rr1>rr0){
      if (rr1>size)
        size=rr1;
      delta=rr1-rr0;
      rr0=rr1;
    }
    mpfr_add(rr,rr,Pi._REALptr->inf,MPFR_RNDN);
  }
  else if (Pi.type==_CPLX){
    mpfr_t & rinf=Pi._CPLXptr->_REALptr->inf;
    mpfr_get_d_2exp(&rr1,rinf,MPFR_RNDN);
    if (rr1>rr0){
      if (rr1>size)
        size=rr1;
      delta=rr1-rr0;
      rr0=rr1;
    }
    mpfr_add(rr,rr,rinf,MPFR_RNDN);
    mpfr_add(ri,ri,(Pi._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
  }
  else if (Pi.type==_INT_){
    mpfr_add_si(rr,rr,Pi.val,MPFR_RNDN);
    return ;
  }
  else exit(1);
  mpfr_get_d_2exp(&rr1,rr,MPFR_RNDN);
  if (delta>nbits)
    delta=nbits;
  if (loss>delta)
    loss -= delta;
  else
    loss = 0;
  if (rr1<rr0){
    loss += rr0-rr1;
  }
}

void mult(mpfr_t & rr,mpfr_t & ri,const gen & x,mpfr_t & tmp,mpfr_t & tmp1, mpfr_t & tmp2){
  if (x.type==_REAL){
    mpfr_mul(rr,rr,x._REALptr->inf,MPFR_RNDN);
    //if (!mpfr_zero_p(ri))
      mpfr_mul(ri,ri,x._REALptr->inf,MPFR_RNDN);
    return;
  }
  if (x.type==_CPLX){
#if 0
    mpfr_fmms(tmp1,rr,x._CPLXptr->_REALptr->inf,ri,(x._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
    mpfr_fmma(ri,rr,(x._CPLXptr+1)->_REALptr->inf,ri,x._CPLXptr->_REALptr->inf,MPFR_RNDN);
    mpfr_swap(tmp1,rr);
    return;
#endif
#if 0
    // (rr+i*ri)*(xr+i*xi)=rr*xr-ri*xi+i*(rr*xi+ri*xr)
    mpfr_mul(tmp1,rr,x._CPLXptr->_REALptr->inf,MPFR_RNDN);
    mpfr_mul(tmp2,ri,(x._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
    // imag part = (rr+ri)*(xr+xi)-(rr*xr+ri*xi)
    mpfr_add(rr,rr,ri,MPFR_RNDN);
    mpfr_add(ri,x._CPLXptr->_REALptr->inf,(x._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
    mpfr_mul(tmp,rr,ri,MPFR_RNDN);
    mpfr_sub(tmp,tmp,tmp1,MPFR_RNDN);
    mpfr_sub(ri,tmp,tmp2,MPFR_RNDN);
    mpfr_sub(rr,tmp1,tmp2,MPFR_RNDN);
    return;
#endif
    mpfr_mul(tmp1,rr,x._CPLXptr->_REALptr->inf,MPFR_RNDN);
    mpfr_mul(tmp2,ri,(x._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
    mpfr_sub(tmp,tmp1,tmp2,MPFR_RNDN);
    mpfr_mul(tmp1,rr,(x._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
    mpfr_mul(tmp2,ri,x._CPLXptr->_REALptr->inf,MPFR_RNDN);
    mpfr_swap(tmp,rr);
    mpfr_add(ri,tmp1,tmp2,MPFR_RNDN);
    return;
  }
  if (x.type==_INT_){
    mpfr_mul_si(rr,rr,x.val,MPFR_RNDN);
    //if (!mpfr_zero_p(ri))
      mpfr_mul_si(ri,ri,x.val,MPFR_RNDN);
    return;
  }
  exit(1);
}

// find r=P(x) and r1=diff(P)(x)
// returns largest number of bits of mantissa in intermediate computations
long horner2_mpfr(const vdbl & P,const dbl & x,dbl & r,dbl & r1,int nbits,long & size,bool pdiff){
  if (P.empty())
    return 0;
  long loss=0;
  size=-RAND_MAX;
  size_t s=P.size()-1;
  mpfr_t rr,ri,r1r,r1i,tmp,tmp1,tmp2;
  mpfr_init2(rr,nbits); mpfr_set_si(rr,0,MPFR_RNDN);
  mpfr_init2(ri,nbits); mpfr_set_si(ri,0,MPFR_RNDN);
  mpfr_init2(r1r,nbits); mpfr_set_si(r1r,0,MPFR_RNDN);
  mpfr_init2(r1i,nbits); mpfr_set_si(r1i,0,MPFR_RNDN);
  mpfr_init2(tmp,nbits);
  mpfr_init2(tmp1,nbits);
  mpfr_init2(tmp2,nbits);
  for (size_t i=0;i<s;++i){
    // r=r*x+P[i];
    mult(rr,ri,x,tmp,tmp1,tmp2);
    add(rr,ri,P[i],nbits,loss,size);
    if (pdiff){
      // r1=r1*x+r;
      mult(r1r,r1i,x,tmp,tmp1,tmp2);
      mpfr_add(r1r,r1r,rr,MPFR_RNDN);
      mpfr_add(r1i,r1i,ri,MPFR_RNDN);
    }
  }
  // r=r*x+P[s];
  mult(rr,ri,x,tmp,tmp1,tmp2);
  add(rr,ri,P[s],nbits,loss,size);
  // end
  r=gen(real_object(rr),real_object(ri));
  if (pdiff)
    r1=gen(real_object(r1r),real_object(r1i));
  mpfr_clear(rr);
  mpfr_clear(ri);
  mpfr_clear(r1r);
  mpfr_clear(r1i);
  mpfr_clear(tmp);
  mpfr_clear(tmp1);
  mpfr_clear(tmp2);
  return loss;
}

long secular_mpfr(const vdbl & A,const vdbl & B,const dbl & x,dbl & r,dbl & r1,int nbits,long & size){
  if (A.empty())
    return 0;
  long loss=0;
  size=-RAND_MAX;
  size_t s=A.size()-1;
  mpfr_t rr,ri,r1r,r1i,tmp,tmp1,tmp2;
  mpfr_init2(rr,nbits); mpfr_set_si(rr,0,MPFR_RNDN);
  mpfr_init2(ri,nbits); mpfr_set_si(ri,0,MPFR_RNDN);
  mpfr_init2(r1r,nbits); mpfr_set_si(r1r,0,MPFR_RNDN);
  mpfr_init2(r1i,nbits); mpfr_set_si(r1i,0,MPFR_RNDN);
  mpfr_init2(tmp,nbits);
  mpfr_init2(tmp1,nbits);
  mpfr_init2(tmp2,nbits);
  for (size_t i=0;i<s;++i){
    // r=r*x+P[i];
    mult(rr,ri,x,tmp,tmp1,tmp2);
    add(rr,ri,A[i],nbits,loss,size);
    // r1=r1*x+r;
    mult(r1r,r1i,x,tmp,tmp1,tmp2);
    mpfr_add(r1r,r1r,rr,MPFR_RNDN);
    mpfr_add(r1i,r1i,ri,MPFR_RNDN);
  }
  // r=r*x+P[s];
  mult(rr,ri,x,tmp,tmp1,tmp2);
  add(rr,ri,A[s],nbits,loss,size);
  // end
  r=gen(real_object(rr),real_object(ri));
  r1=gen(real_object(r1r),real_object(r1i));
  mpfr_clear(rr);
  mpfr_clear(ri);
  mpfr_clear(r1r);
  mpfr_clear(r1i);
  mpfr_clear(tmp);
  mpfr_clear(tmp1);
  mpfr_clear(tmp2);
  return loss;
}
#endif

#ifdef LDBL80
bool convert(const gen & g,long double & z){
  if (g.type==_INT_){
    z=g.val;
    return true;
  }
  if (g.type==_DOUBLE_){
    z=g._DOUBLE_val;
    return true;
  }
  if (g.type==_REAL){
#ifdef HAVE_LIBMPFR
    z=mpfr_get_ld(g._REALptr->inf,MPFR_RNDN);
#else
    z=mpf_get_d(g._REALptr->inf);
#endif
    return true;
  }
  if (g.type==_FRAC){
    long double n,d;
    if (convert(g._FRACptr->num,n) && convert(g._FRACptr->den,d)){
      z=n/d;
      return true;
    }
    return false;
  }
  if (g.type!=_ZINT)
    return false;
  int s=mpz_cmp_si(*g._ZINTptr,0);
  int l=mpz_sizeinbase(*g._ZINTptr,2);
  if (l>=(1<<15))
    return false;
  mpz_t zz; mpz_init(zz);
  if (l>64)
    // we have 64 bits of mantissa
    mpz_div_2exp(zz,*g._ZINTptr,l-64);
  else
    mpz_set(zz,*g._ZINTptr);
  ulonglong u;
  u=mpz_get_ui(zz);
  mpz_clear(zz);
  z=u;
  if (l>64)
    z=z*pow((long double) 2,l-64);
  if (s<0)
    z=-z;
  return true;
}
#endif
      
bool convert(const vdbl & P,vfdbl & fP,GIAC_CONTEXT){
  int s=P.size();
  fP.clear(); fP.reserve(s);
  for (int i=0;i<s;++i){
    dbl real,imag;
    reim(P[i],real,imag,contextptr);
#ifdef LDBL80
    long double zr,zi;
    if (!convert(real,zr) || !convert(imag,zi))
      return false;
    fP.push_back(fdbl(zr,zi));
#else
    if (real.type==_ZINT && mpz_sizeinbase(*real._ZINTptr,2)>1000)
      return false;
    if (imag.type==_ZINT && mpz_sizeinbase(*imag._ZINTptr,2)>1000)
      return false;
    fP.push_back(fdbl(evalf_double(real,1,contextptr)._DOUBLE_val,evalf_double(imag,1,contextptr)._DOUBLE_val));
#endif
  }
  return true;
}

void convert(const vfdbl & fP,vdbl & P){
  int s=fP.size();
  P.clear(); P.reserve(s);
  for (int i=0;i<s;++i){
#if defined HAVE_LIBMPFR && defined LDBL80
    dbl re,im;
    re=accurate_evalf(re,64); im=accurate_evalf(im,64);
    mpfr_set_ld(re._REALptr->inf,fP[i].real(),MPFR_RNDN);
    mpfr_set_ld(im._REALptr->inf,fP[i].imag(),MPFR_RNDN);
#else
    dbl re(double(fP[i].real())),im(double(fP[i].imag()));    
#endif
    P.push_back(dbl(re,im));
  }
}

double l1norm(const vdbl & v){
  double r=0;
  for (int i=0;i<v.size();++i)
    r += absdbl(v[i]);
  return r;
}

void div(vdbl & v,double r){
  r=1/r;
  for (int i=0;i<v.size();++i)
    v[i] = r*v[i];
}

#if 0
// approx largest root in norm, using a few iterations of power method
double largest(const vdbl & fP,int maxiter=100,double eps=1e-6){
  int deg=fP.size()-1;
  vdbl v(deg);
  for (int i=0;i<deg;++i)
    v[i]=dbl(rand()/(1.0+RAND_MAX),rand()/(1.0+RAND_MAX));
  div(v,l1norm(v));
  vdbl w(deg);
  double oldR(0.0);
  for (int j=0;j<maxiter;++j){
    for (int i=0;i<deg-1;++i){
      w[i+1]=v[i];
    }
    dbl r(0);
    for (int i=0;i<deg;++i){
      r += fP[i+1]*v[i];
    }
    w[0]= -r/fP[0];
    double R=l1norm(w);
    if (debug_infolevel)
      *logptr(contextptr) << "power method " << j << " " << R << "\n";
    div(w,R);
    if (j>5 && abs(1-R/oldR)<eps)
      return R;
    v.swap(w);
    oldR=R;
  }
  return oldR;
}

#else
double largest(const vdbl & P,int maxiter=100,double eps=1e-6){
  vfdbl fP; convert(P,fP,context0);
  return largest(fP,maxiter,eps);
}
#endif

double cross_prod(const vdbl & v,int a,int b,int c){
  dbl ab=v[b]-v[a],ac=v[c]-v[a];
  double A=redbl(ab),B=imdbl(ab),C=redbl(ac),D=imdbl(ac);
  return A*D-B*C;
}

vector<int> convexhull(const vdbl & v){
  int s=v.size(),imin=0;
  if (s==1)
    return vector<int>(1,0);
  // find origin
  double ymin=imdbl(v[0]),ycur,xmin=redbl(v[0]),xcur;
  for (int i=1;i<s;++i){
    ycur=imdbl(v[i]);xcur=redbl(v[i]);
    if (ymin>ycur || (ymin==ycur && xmin>xcur) ){
      imin=i; ymin=ycur; xmin=xcur;
    }
  }
  vector<int_2double> ls;
  for (int j=0;j<s;++j){
    if (j!=imin){
      double dx=redbl(v[j])-xmin,dy=imdbl(v[j])-ymin;
      int_2double s={j,atan2(dy,dx),norm(dx,dy)};
      ls.push_back(s);
    }
  }
  sort(ls.begin(),ls.end(),graham_sort_function);
  vector<int> res; res.push_back(imin); res.push_back(ls[0].i);
  int ress=2;
  for (int j=1;j<s-1;++j){
    int icur=ls[j].i;
    double o=cross_prod(v,res[ress-2],res[ress-1],icur);
    if (o==0)
      res[ress-1]=icur;
    else {
      if (o>0){
        res.push_back(icur);
        ++ress;
      }
      else {
        while (ress>2 && o<0){
          res.pop_back();
          ress--;
          o=cross_prod(v,res[ress-2],res[ress-1],icur);
        }
        res.push_back(icur);
        ++ress;
      }
    }
  }
  return res;
}

void init_R(const vdbl & P,vdbl & R){
  R.clear();
  int n=P.size()-1;
  vdbl l;
  vector<int> lpos;
  for (int i=0;i<=n;++i){
    double ai=logabsdbl(P[n-i]);
    if (ai==MINREAL)
      continue;
    l.push_back(dbl(double(i),ai));
    lpos.push_back(i);
  }
  vector<int> pos=convexhull(l);
  // find real positions (since coeffs==0 were removed)
  for (int i=0;i<pos.size();++i){
    pos[i]=lpos[pos[i]];
  }
  sort(pos.begin(),pos.end());
  if (pos[0]!=0)
    pos.insert(pos.begin(),0);
  if (pos.back()!=n)
    pos.push_back(n);
  int count=0;
  vector<int_double> dkuk;
  for (int i=1;i<pos.size();++i){
    int dk=pos[i]-pos[i-1];
    const dbl & tmp =P[n-pos[i-1]];
    double uk=is_exactly_zero(tmp)?0:exp(-logabsdbl(P[n-pos[i]]/tmp)/dk);//pow(absdbl(P[n-pos[i]]/P[n-pos[i-1]]),-1.0/dk);
    int_double id={dk,uk};
    dkuk.push_back(id);
  }
  sort(dkuk.begin(),dkuk.end(),norm_sort);
  for (int i=0;i<dkuk.size();++i){
    int dk=dkuk[i].i;
    double uk=dkuk[i].d;
    double sigma=0.7;
    for (int j=0;j<dk;++j){
      double theta=sigma+2*M_PI*i/n+2*M_PI*j/dk;
      dbl z(uk*cos(theta),uk*sin(theta));
      R.push_back(z);
    }
  }
}

bool sum_inv_diff(const gen & zi,const vdbl & R,int i,gen & p){
  p=0;
  for (int j=0;j<R.size();++j){
    if (j==i) continue;
    dbl dz=(zi-R[j]);
    if (is_exactly_zero(dz))
      return false;
    p += inv(dz);
  }
  return true;
}

#ifdef HAVE_LIBMPFR
void sub(mpfr_t & rr,mpfr_t & ri,const gen & Pi){
  if (Pi.type==_CPLX){
    mpfr_sub(rr,rr,Pi._CPLXptr->_REALptr->inf,MPFR_RNDN);
    mpfr_sub(ri,ri,(Pi._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
  }
  else if (Pi.type==_REAL)
    mpfr_sub(rr,rr,Pi._REALptr->inf,MPFR_RNDN);
  else if (Pi.type==_INT_)
    mpfr_sub_si(rr,rr,Pi.val,MPFR_RNDN);
  else exit(1);
}

void inv(mpfr_t & r,mpfr_t & i,mpfr_t & tmp,mpfr_t & tmp1, mpfr_t & tmp2){
  if (mpfr_zero_p(i)){
    mpfr_set_si(tmp,1,MPFR_RNDN);
    mpfr_div(r,tmp,r,MPFR_RNDN);
  }
  else {
    mpfr_sqr(tmp1,r,MPFR_RNDN);
    mpfr_sqr(tmp2,i,MPFR_RNDN);
    mpfr_add(tmp,tmp1,tmp2,MPFR_RNDN);
#if 0
    mpfr_ui_div(tmp1,1,tmp,MPFR_RNDN);
    mpfr_mul(r,r,tmp1,MPFR_RNDN);
    mpfr_neg(tmp1,tmp1,MPFR_RNDN);
    mpfr_mul(i,i,tmp1,MPFR_RNDN);
#else
    mpfr_div(r,r,tmp,MPFR_RNDN);
    mpfr_neg(tmp,tmp,MPFR_RNDN);
    mpfr_div(i,i,tmp,MPFR_RNDN);
#endif
  }
}

bool mpfr_sum_inv_diff(const gen & z,const vdbl & R,int i,gen & p,int nbits,GIAC_CONTEXT){
  mpfr_t tmp,tmp1,tmp2,zr,zi,pr,pi;
  mpfr_init2(pr,nbits); mpfr_set_si(pr,0,MPFR_RNDN);
  mpfr_init2(pi,nbits); mpfr_set_si(pi,0,MPFR_RNDN);
  mpfr_init2(zr,nbits); 
  mpfr_init2(zi,nbits); 
  mpfr_init2(tmp,nbits);
  mpfr_init2(tmp1,nbits);
  mpfr_init2(tmp2,nbits);
  for (int j=0;j<R.size();++j){
    if (j==i) continue;
    // dbl dz=(z-R[j]);
    if (z.type==_REAL){
      mpfr_set(zr,z._REALptr->inf,MPFR_RNDN);
      mpfr_set_si(zi,0,MPFR_RNDN);
    }
    else if (z.type==_CPLX){
      mpfr_set(zr,z._CPLXptr->_REALptr->inf,MPFR_RNDN);
      mpfr_set(zi,(z._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
    }
    else if (z.type==_INT_){
      mpfr_set_si(zr,z.val,MPFR_RNDN);
      mpfr_set_si(zi,0,MPFR_RNDN);
    }
    else exit(1);
    sub(zr,zi,R[j]); 
    if (mpfr_zero_p(zr) && mpfr_zero_p(zi)){
      mpfr_clear(pr);
      mpfr_clear(pi);
      mpfr_clear(zr);
      mpfr_clear(zi);
      mpfr_clear(tmp);
      mpfr_clear(tmp1);
      mpfr_clear(tmp2);
      if (debug_infolevel>1)
        *logptr(contextptr) << "sum_inv_diff equal R index i=" << i << ", j=" << j << ", z=" << z << "\n";
      return false;
    }
    inv(zr,zi,tmp,tmp1,tmp2);
    // p += inv(dz);
    mpfr_add(pr,pr,zr,MPFR_RNDN);
    mpfr_add(pi,pi,zi,MPFR_RNDN);
  }
  p=gen(real_object(pr),real_object(pi));
  mpfr_clear(pr);
  mpfr_clear(pi);
  mpfr_clear(zr);
  mpfr_clear(zi);
  mpfr_clear(tmp);
  mpfr_clear(tmp1);
  mpfr_clear(tmp2);  
  return true;
}

// a/z where re(z)==r and im(z)==i
void div(const gen & a,mpfr_t & r,mpfr_t & i,mpfr_t & tmp,mpfr_t & tmp1, mpfr_t & tmp2,mpfr_t & R,mpfr_t & I){
  mpfr_set(R,r,MPFR_RNDN);
  mpfr_set(I,i,MPFR_RNDN);
  mpfr_neg(i,i,MPFR_RNDN);
  mult(r,i,a,tmp,tmp1,tmp2);
  // (tmp1+i*tmp2)/(R+i*I) = (tmp1+i*tmp2)*(R-i*I)/(R^2+I^2)
  mpfr_sqr(tmp1,R,MPFR_RNDN);
  mpfr_sqr(tmp2,I,MPFR_RNDN);
  mpfr_add(tmp,tmp1,tmp2,MPFR_RNDN);
  mpfr_div(r,r,tmp,MPFR_RNDN);
  mpfr_div(i,i,tmp,MPFR_RNDN);
}

// sum(a_i/(z-b_i))
bool mpfr_node_sum(const gen & z,const vdbl & A,const vdbl & B,gen & p,int nbits,GIAC_CONTEXT){
  mpfr_t tmp,tmp1,tmp2,zr,zi,pr,pi,R,I;
  mpfr_init2(pr,nbits); mpfr_set_si(pr,0,MPFR_RNDN);
  mpfr_init2(pi,nbits); mpfr_set_si(pi,0,MPFR_RNDN);
  mpfr_init2(zr,nbits); 
  mpfr_init2(zi,nbits); 
  mpfr_init2(R,nbits); 
  mpfr_init2(I,nbits); 
  mpfr_init2(tmp,nbits);
  mpfr_init2(tmp1,nbits);
  mpfr_init2(tmp2,nbits);
  for (int j=0;j<B.size();++j){
    // dbl dz=(z-R[j]);
    if (z.type==_REAL){
      mpfr_set(zr,z._REALptr->inf,MPFR_RNDN);
      mpfr_set_si(zi,0,MPFR_RNDN);
    }
    else if (z.type==_CPLX){
      mpfr_set(zr,z._CPLXptr->_REALptr->inf,MPFR_RNDN);
      mpfr_set(zi,(z._CPLXptr+1)->_REALptr->inf,MPFR_RNDN);
    }
    else if (z.type==_INT_){
      mpfr_set_si(zr,z.val,MPFR_RNDN);
      mpfr_set_si(zi,0,MPFR_RNDN);
    }
    else exit(1);
    sub(zr,zi,B[j]); 
    if (mpfr_zero_p(zr) && mpfr_zero_p(zi)){
      mpfr_clear(pr);
      mpfr_clear(pi);
      mpfr_clear(zr);
      mpfr_clear(zi);
      mpfr_clear(R);
      mpfr_clear(I);
      mpfr_clear(tmp);
      mpfr_clear(tmp1);
      mpfr_clear(tmp2);
      if (debug_infolevel>1)
        *logptr(contextptr) << "mpfr_node_sum equal B index j=" << j << ", z=" << z << "\n";
      return false;
    }
    div(A[j],zr,zi,R,I,tmp,tmp1,tmp2);
    // p += inv(dz);
    mpfr_add(pr,pr,zr,MPFR_RNDN);
    mpfr_add(pi,pi,zi,MPFR_RNDN);
  }
  p=gen(real_object(pr),real_object(pi));
  mpfr_clear(pr);
  mpfr_clear(pi);
  mpfr_clear(zr);
  mpfr_clear(zi);
  mpfr_clear(R);
  mpfr_clear(I);
  mpfr_clear(tmp);
  mpfr_clear(tmp1);
  mpfr_clear(tmp2);  
  return true;
}
#endif

// check that roots are isolated or have precision eps, set zi_done[i] to 2 or 0
bool chk_isol(vdbl & z,bool realpoly,int isolate,const vdbl & rz,double eps,vector<short int> & zi_done,GIAC_CONTEXT){
  int deg=z.size();
  if (debug_infolevel && isolate)
    *logptr(contextptr) << "Certifying roots begin " << CLOCK()*1e-6 << "\n";
  dbl big(1LL<<62); // take care of almost complete cancellation in -
  for (int i=0;i<deg;++i){
    dbl & zi=z[i];
    // dbl err=abs(zi)/big;
    const dbl & rzi=rz[i]; // +err
    if (1 || zi_done[i]){
      zi_done[i]=2;
      for (int j=i+1;j<deg;++j){
        dbl zij=sqrt((z[j]-zi).squarenorm(contextptr),contextptr);
        if (is_greater(rzi+rz[j],zij,contextptr))
          zi_done[i]=zi_done[j]=0;
      }
    }
    if (zi_done[i] && realpoly && zi.type==_CPLX){
      dbl I=*(zi._CPLXptr+1);
      dbl c=conj(zi,contextptr);
      bool maybereal=is_positive(rzi-I,contextptr);
      int nconj=0; // 0 after loop if root is real
      bool unique=true;
      for (int j=i+1;j<deg;++j){
        dbl zij=(z[j]-c);
        // Note that zij may be = to abs(c)*2^-63*sqrt(2) if we loose all precision
        zij=rzi+rz[j]-abs(zij,contextptr);
        if (is_positive(zij,contextptr)){
          zi_done[i]=4; // potential conjugate
          if (nconj==0)
            nconj=j;
          else
            unique=false; // more than one potential conjugate
        }
      }
      if (nconj==0){
        if (maybereal) // clean it
          zi=*zi._CPLXptr;
      }
      else {
        swap(z[i+1],z[nconj]);
        swap(zi_done[i+1],zi_done[nconj]);
        zi_done[i]=4;
        if (unique){
          zi_done[i+1]=5;
          z[i+1]=c;
        }
        ++i;
      }
    }
  }
  // isolate==1 if goal is isolation only
  if (isolate!=1){ 
    bool approx=true;
    for (int i=0;i<deg;++i){
      dbl & zi=z[i];
      // dbl err=abs(zi)/big;
      const dbl & rzi=rz[i]; // +err
      if (zi_done[i]!=5){
        gen chk=is_exactly_zero(zi)?eps:eps*abs(zi,contextptr);
        if (is_greater(rzi,chk,contextptr)){
          zi_done[i]=0;
          approx=false;
        }
      }
    }
    if (!isolate)
      return approx;
  }
  for (int i=0;i<deg;++i){
    if (!zi_done[i]){
      if (debug_infolevel)
        *logptr(contextptr) << "Failure certifying roots " << CLOCK()*1e-6 << "\n";
      return false;
    }
  }
  if (debug_infolevel)
    *logptr(contextptr) << "Certifying roots end " << CLOCK()*1e-6 << "\n";
  return true;
}

void clear_zi_done(vector<short int> & zi_done){
  for (int i=0;i<zi_done.size();++i){
    if (zi_done[i]!=5)
      zi_done[i]=0;
  }
}

#if defined HAVE_LIBMPFI && !defined NO_RTTI
bool maybe_zero(const dbl & g){
  dbl a=abs(g,context0);
  if (a.type==_REAL){
    if (real_interval * ptr=dynamic_cast<real_interval *>(a._REALptr))
      return ptr->maybe_zero();
  }
  return is_exactly_zero(a);
}
void add(mpfi_t & rr,mpfi_t & ri,const gen & Pi){
  if (Pi.type==_CPLX){
    mpfi_add(rr,rr,dynamic_cast<real_interval *>(Pi._CPLXptr->_REALptr)->infsup);
    mpfi_add(ri,ri,dynamic_cast<real_interval *>((Pi._CPLXptr+1)->_REALptr)->infsup);
  }
  else if (Pi.type==_REAL)
    mpfi_add(rr,rr,dynamic_cast<real_interval *>(Pi._REALptr)->infsup);
  else if (Pi.type==_INT_)
    mpfi_add_si(rr,rr,Pi.val);
  else exit(1);
}

void mult(mpfi_t & rr,mpfi_t & ri,const gen & x,mpfi_t & tmp,mpfi_t & tmp1, mpfi_t & tmp2){
  if (x.type==_CPLX){
    mpfi_mul(tmp1,rr,dynamic_cast<real_interval *>(x._CPLXptr->_REALptr)->infsup);
    mpfi_mul(tmp2,ri,dynamic_cast<real_interval *>((x._CPLXptr+1)->_REALptr)->infsup);
    mpfi_sub(tmp,tmp1,tmp2);
    mpfi_mul(tmp1,rr,dynamic_cast<real_interval *>((x._CPLXptr+1)->_REALptr)->infsup);
    mpfi_mul(tmp2,ri,dynamic_cast<real_interval *>(x._CPLXptr->_REALptr)->infsup);
    mpfi_swap(tmp,rr);
    mpfi_add(ri,tmp1,tmp2);
  }
  else if (x.type==_REAL){
    mpfi_mul(rr,rr,dynamic_cast<real_interval *>(x._REALptr)->infsup);
    mpfi_mul(ri,ri,dynamic_cast<real_interval *>(x._REALptr)->infsup);
  }
  else if (x.type==_INT_){
    mpfi_mul_si(rr,rr,x.val);
    mpfi_mul_si(ri,ri,x.val);
  }
  else exit(1);
}

// find r=P(x) and r1=diff(P)(x)
bool horner2_mpfi(const vdbl & P,dbl x,dbl & r,dbl & r1,int nbits,bool pdiff=true){
  if (P.empty())
    return false;
  size_t s=P.size()-1;
  mpfi_t rr,ri,r1r,r1i,tmp,tmp1,tmp2;
  mpfi_init2(rr,nbits); mpfi_set_si(rr,0);
  mpfi_init2(ri,nbits); mpfi_set_si(ri,0);
  mpfi_init2(r1r,nbits); mpfi_set_si(r1r,0);
  mpfi_init2(r1i,nbits); mpfi_set_si(r1i,0);
  mpfi_init2(tmp,nbits);
  mpfi_init2(tmp1,nbits);
  mpfi_init2(tmp2,nbits);
  for (size_t i=0;i<s;++i){
    // r=r*x+P[i];
    mult(rr,ri,x,tmp,tmp1,tmp2);
    add(rr,ri,P[i]);
    if (pdiff){
      // r1=r1*x+r;
      mult(r1r,r1i,x,tmp,tmp1,tmp2);
      mpfi_add(r1r,r1r,rr);
      mpfi_add(r1i,r1i,ri);
    }
  }
  // r=r*x+P[s];
  mult(rr,ri,x,tmp,tmp1,tmp2);
  add(rr,ri,P[s]);
  // end
  r=gen(real_interval(rr),real_interval(ri));
  r1=gen(real_interval(r1r),real_interval(r1i));
  mpfi_clear(rr);
  mpfi_clear(ri);
  mpfi_clear(r1r);
  mpfi_clear(r1i);
  mpfi_clear(tmp);
  mpfi_clear(tmp1);
  mpfi_clear(tmp2);
  return true;
}


// find r=P(x) and r1=diff(P)(x)
bool horner2_mpfi(const vdbl & P,const vdbl & Pdiff,dbl x,dbl & r,dbl & r1,int nbits){
  if (P.empty())
    return false;
  size_t s=P.size()-1;
  mpfi_t rr,ri,r1r,r1i,tmp,tmp1,tmp2;
  mpfi_init2(rr,nbits); mpfi_set_si(rr,0);
  mpfi_init2(ri,nbits); mpfi_set_si(ri,0);
  mpfi_init2(r1r,nbits); mpfi_set_si(r1r,0);
  mpfi_init2(r1i,nbits); mpfi_set_si(r1i,0);
  mpfi_init2(tmp,nbits);
  mpfi_init2(tmp1,nbits);
  mpfi_init2(tmp2,nbits);
  for (size_t i=0;i<s;++i){
    // r=r*x+P[i];
    mult(rr,ri,x,tmp,tmp1,tmp2);
    add(rr,ri,P[i]);
    // r1=r1*x+Pdiff[i];
    mult(r1r,r1i,x,tmp,tmp1,tmp2);
    add(r1r,r1i,Pdiff[i]);
  }
  // r=r*x+P[s];
  mult(rr,ri,x,tmp,tmp1,tmp2);
  add(rr,ri,P[s]);
  // end
  r=gen(real_interval(rr),real_interval(ri));
  r1=gen(real_interval(r1r),real_interval(r1i));
  mpfi_clear(rr);
  mpfi_clear(ri);
  mpfi_clear(r1r);
  mpfi_clear(r1i);
  mpfi_clear(tmp);
  mpfi_clear(tmp1);
  mpfi_clear(tmp2);
  return true;
}

#else
bool maybe_zero(const dbl & g){
  return is_exactly_zero(g);
}

#endif

#ifdef HAVE_LIBMPFR
// returns 0 on failure (e.g. division by maybe 0), -1 too many iterations, 1 ok, 2 isolated
int aberth_mpfr(const vdbl & P0,bool realpoly,int & nbits,int N,double eps,vdbl & R,vdbl & rayon,int cluster_start,int cluster_afterend,vector<short int> & zi_done,bool certify_lastiter,int isolate,bool secular,GIAC_CONTEXT){
  int afteriter=2;
  int deg=P0.size()-1,prevcount=0;
  bool doing_cluster=cluster_start>0 || cluster_afterend<deg;
  vdbl P(P0);
  dbl dr(0);
  dbl l(1);
  if (R.size()!=deg){
    dr=-find_shift(P,contextptr);
    P=shift(P,dr);
    //l=find_scale(P);
    //rescale(P,l);
    // init R if not already done, using single precision Aberth
    vfdbl fP,fR;
    if (convert(P,fP,contextptr)){
      double eps2=1e-4;
      bool ok=aberth_singleprec(fP,2*N,eps2,fR,0,deg,zi_done,4,contextptr);
      convert(fR,R);
      if (ok &&
	  !doing_cluster){
        vdbl P_cert(P);
        int nbits=64;
        accurate_evalf(P_cert,nbits);
#if defined MPFI_CERT && defined HAVE_LIBMPFI && !defined NO_RTTI
        P_cert=*convert_interval(P_cert,nbits,contextptr)._VECTptr;
#endif
        // find rayon with exact computation or MPFI
        int bits=nbits;
        vdbl P1(P);
        accurate_evalf(P1,bits);
        for (int i=0;ok && i<deg;++i){
#if 1 // def HAVE_LIBMPFR
          gen z=accurate_evalf(R[i],nbits),d,d1;
#if defined MPFI_CERT && defined HAVE_LIBMPFI && !defined NO_RTTI
          z=convert_interval(z,nbits,contextptr);
          horner2_mpfi(P_cert,convert_interval(z,nbits,contextptr),d,d1,nbits);
          if (maybe_zero(d1)){
            ok=false;
          }
          else {
            d=d/d1;
            rayon[i]=deg*abs(d,contextptr);
            if (rayon[i].type==_REAL)
              rayon[i]=_right(rayon[i],contextptr);
          }
#else // MPFI
          for (;;){
            long size; int loss=horner2_mpfr(P1,z,d,d1,bits,size,true);
            if (loss<bits-16){
              rayon[i]=abs(accurate_evalf(d/d1,nbits),contextptr);
              break;
            }
            bits*=2;
            z=accurate_evalf(R[i],bits);
            P1=P;
            accurate_evalf(P1,bits);
          }
#endif // MPFI
#else // MPFR
          gen z=exact(R[i],contextptr),d,d1;
          horner2(P,z,d,d1);
          rayon[i]=abs(evalf_double(d/d1,1,contextptr));
#endif
          if (is_greater(rayon[i],eps*abs(R[i],contextptr),contextptr))
            ok=false;
        }
        // early termination in single precision?
        if (ok &&
            chk_isol(R,realpoly,isolate,rayon,eps,zi_done,contextptr)
            ){
          for (size_t j=0;j<R.size();++j){
            R[j] += dr;
            if (!isolate && eps>1e-14)
              R[j]=evalf_double(R[j],1,contextptr);
          }
          return 2;
        }
      }
    }
    if (R.empty())
      init_R(P,R);
  }
  accurate_evalf(P,nbits);
  clear_zi_done(zi_done);
  vdbl P_cert; int nbitscert=nbits;
  if (certify_lastiter){
#if defined MPFI_CERT && defined HAVE_LIBMPFI && !defined NO_RTTI
    P_cert=*convert_interval(P,nbitscert,contextptr)._VECTptr;
#else
    P_cert=P;
#endif
  }
  accurate_evalf(R,nbits);
  vdbl P2(P0); int nbitsP2=2*nbits+64; 
  accurate_evalf(P2,nbitsP2);
#if 0
  vdbl P_cert_diff(derivative(P0)); 
  if (certify_lastiter){
#if 0 // defined MPFI_CERT && defined HAVE_LIBMPFI && !defined NO_RTTI
    P_cert_diff=*convert_interval(P_cert_diff,nbits,contextptr)._VECTptr;
#else
    accurate_evalf(P_cert_diff,nbits);
#endif
  }
#endif
  vdbl newR(R); // init with R is required in clusters
  int ok=0; long size; double delta; bool isol;
  vdbl A,B; // for secular equation-like eval
  for (int k=0;k<N;++k){
    if (debug_infolevel>0)
      *logptr(contextptr) << clock()*1e-6 << " Aberth bits=" << nbits << " iter="<<  k << " ";
    delta=0; isol=true;
    long maxloss=0,minloss=0;
    if (doing_cluster){
      int K=cluster_start;
      for (;K<cluster_afterend;++K){
        if (!zi_done[K])
          break;
      }
      if (K==cluster_afterend)
        ok=1;
    }
    for (int i=cluster_start;i<cluster_afterend;++i){
      if (i && zi_done[i]==5){
        newR[i]=R[i]=conj(R[i-1],contextptr);
        rayon[i]=rayon[i-1];
        continue;
      }
      if (zi_done[i]>=2 ||
          (!ok && zi_done[i])
          ){
        // if root is not isolated, do the last iteration
        newR[i]=R[i];
        continue;
      }
      dbl zi=R[i];
      if (debug_infolevel>2)
        *logptr(contextptr) << CLOCK()*1e-6 << " computing d\n"; 
      dbl d,d1;
      long loss=0;
      if (ok && certify_lastiter){
#if defined MPFI_CERT && defined HAVE_LIBMPFI && !defined NO_RTTI
        int loss2=RAND_MAX;
        if (isolate==0)
          loss2=horner2_mpfr(P2,accurate_evalf(zi,nbitsP2),d,d1,2*nbits,size,true);
        if (loss2>=nbitsP2-16){
          gen zicert=convert_interval(zi,nbits,contextptr),dd,dd1;        
          horner2_mpfi(P_cert,zicert,d,d1,nbits);
          if (maybe_zero(d1)){
            if (debug_infolevel)
              *logptr(contextptr) << "MPFI unable to certify radius, root " << i << zi << "\n";
            return -1;
          }
        }
#else
        loss=horner2_mpfr(P_cert,accurate_evalf(zi,nbitscert),d,d1,nbitscert,size,true);
        while (loss>nbitscert-16){
          nbitscert*=2;
          P_cert=P0;
          accurate_evalf(P_cert,nbitscert);
          loss=horner2_mpfr(P_cert,accurate_evalf(zi,nbitscert),d,d1,nbitscert,size,true);
        }
        if (is_exactly_zero(d1))
          return -1;
#endif
      }          
      else {
        if (secular)
          loss=secular_mpfr(A,B,zi,d,d1,nbits,size);
        else
          loss=horner2_mpfr(P,zi,d,d1,nbits,size,true);
        if (is_exactly_zero(d1))
          return -1;
      }
      d=d/d1;
      rayon[i]=deg*abs(d,contextptr);
      if (ok && certify_lastiter && rayon[i].type==_REAL){
#if defined MPFI_CERT && defined HAVE_LIBMPFI && !defined NO_RTTI        
        rayon[i]=_right(rayon[i],contextptr);
        d=gen(_milieu(re(d,contextptr),contextptr),_milieu(im(d,contextptr),contextptr));
#else
        rayon[i]=accurate_evalf(rayon[i],nbits);
#endif
      }
      dbl p(0);
      if (debug_infolevel>2)
        *logptr(contextptr) << CLOCK()*1e-6 << " computing p\n"; 
      bool binv=mpfr_sum_inv_diff(zi,R,i,p,nbits,contextptr);
      if (debug_infolevel>2)
        *logptr(contextptr) << CLOCK()*1e-6 << " end computing p\n"; 
      if (!binv && doing_cluster){
        *logptr(contextptr) << "Root estimates collision \n"; return 0; }
#if 0
      dbl pc(0); sum_inv_diff(zi,R,i,pc);
      if (p!=pc)
        *logptr(contextptr) << "err\n";
#endif
      double dd=absdbl(d),pp=absdbl(p);
      double abszi=absdbl(zi);
      if (debug_infolevel>2)
        *logptr(contextptr) << CLOCK()*1e-6 << " cluster? i=" << i << ", delta=" << dd << ", cluster_step=" << cluster_step << ", p=" << pp << ", d*p=" << dd*pp << " " << cluster_dp << "\n";
      if (!binv ||
          (!ok && k>N/2 && !doing_cluster && dd<=cluster_step*abszi && dd*pp>=cluster_dp)){
        isol=false;
        // cluster of roots, find all roots in this cluster
        int cend=i+1;
        dbl sumR=zi;
        for (int k=cend;k<deg;++k){
          if (absdbl(zi-R[k])<2*deg/pp){
            swap(R[cend],R[k]);
            swap(zi_done[cend],zi_done[k]);
            sumR+=R[cend];
            ++cend;
            if (k<deg-1 && zi_done[k+1]==5)
              zi_done[k+1]=0;
          }
        }
        if (cend-i==deg){
          if (debug_infolevel)
            *logptr(contextptr) << "All estimates are close together\n";
        }
        if (cend-i>1){
          int nbits2=2*nbits,Nc=cend-i;
          // cluster is from i to cend-1 included
          dbl z=sumR/dbl(Nc); // center of gravity of cluster
          // improve z with a Newton iteration on the derivative of order size of cluster -1
          vdbl Pdiff;
          --Nc;
          for (int l=0;l<=deg-Nc;++l){
            gen h=1;
            for (int k=deg-l;k>deg-l-Nc;--k){
              h=k*h;
            }
            Pdiff.push_back(h*P[l]);
          }
          gen z1,z2;
          horner2_mpfr(Pdiff,z,z1,z2,nbits,size,true);
          if (!is_exactly_zero(z2))
            z=z-z1/z2; 
          // shift P and reverse (roots become inverse of roots)
          vdbl Pcluster(P),Rcluster(deg),initR(deg);
          accurate_evalf(Pcluster,nbits2);
          z=accurate_evalf(z,nbits2);
          Pcluster=shift(Pcluster,z,false);
          reverse(Pcluster.begin(),Pcluster.end());
          if (is_exactly_zero(Pcluster[0])){
            Pcluster[0]=z/pow(2,nbits2,contextptr);
          }
          init_R(Pcluster,initR);
          // dbl D(dbl(cend-i)/d);
          int count=deg-1;
          for (int k=0;k<deg;++k){
            if (k>=i && k<cend){
              Rcluster[k]=initR[count];
              --count;
              //dbl a(0.0,0.7+(k-i)*2*M_PI/(cend-i));
              //Rcluster[k]=D*exp(a);
            }
            else
              Rcluster[k]=inv(R[k]-z);
          }
          vector<short int> old(zi_done);
          // recursive call of aberth
          if (debug_infolevel)
            *logptr(contextptr) << "aberth mpfr cluster=" << i << "," << cend << "\n";
          int b=aberth_mpfr(Pcluster,false,nbits2,N,eps,Rcluster,rayon,i,cend,zi_done,false,false,false,contextptr);
          for (int k=i;k<cend;++k){
            R[k]=newR[k]=accurate_evalf(inv(Rcluster[k])+z,nbits);
          }
          if (b){
            i=cend-1;
            //++afteriter;
            continue;
          }
          return -1;
        }
      }
      double absdz=abszi==0?absdbl(d):absdbl(d)/abszi;
      if (binv){
        d=d/(dbl(1)-d*p);
      }
      else {
        isol=false;
        if (ok)
          return 0;
        d=d/(1-rand()/2.0/RAND_MAX);
      }
      delta += absdz;
      newR[i]=zi-d;
      R[i]=newR[i]; // comment to avoid immediate update of the root
      if (absdz<eps/deg){
        if (!zi_done[i]){
          if (debug_infolevel>1)
            *logptr(contextptr) << "New root found " << newR[i] << "\n";
          if (debug_infolevel)
            *logptr(contextptr) << "[" << i << "] ";
        }
        zi_done[i]=1;
      }
      else {
        if (loss<nbits){
          if (debug_infolevel)
            *logptr(contextptr) << i << " ";
        }
        else if (!doing_cluster)
          zi_done[i]=-1;
        if (maxloss<loss)
          maxloss=loss;
        if (minloss==0)
          minloss=loss;
        if (minloss>loss)
          minloss=loss;
      }        
    }
    newR.swap(R);
    int count=0,skipped=0;
    for (int k=0;k<deg;++k){
      if (zi_done[k]>0)
        ++count;
      else if (zi_done[k]==-1)
        ++skipped;      
    }
    if (debug_infolevel>0){
      *logptr(contextptr) << " found=" << count << " skipped " << skipped << " delta=" << delta <<   " precision loss " << double(minloss)/nbits << " (" <<  minloss << "/" << nbits << "), cluster_search=" << cluster_start <<  ".." <<cluster_afterend << " time=" << CLOCK()*1e-6 << "\n";
      if (debug_infolevel>2){
        vdbl RR(R);
        accurate_evalf(RR,45);
        *logptr(contextptr) << "Current approx roots " << RR << "\n";
      }
    }
    if (doing_cluster){
      if (ok)
        return 1;
      continue;
    }
    if (
        delta<=eps*(ok?deg:deg-count) || count==deg
        ){
      if (debug_infolevel)
        *logptr(contextptr) << CLOCK()*1e-6 << " Aberth all roots found " << delta << "\n";      
      ++ok;
    }
    // else ok=0;
    if (ok==afteriter){
      // rescale and translate
      for (size_t j=0;j<R.size();++j){
        R[j] = l*R[j] + dr;
      }
      return 1;
    }
    if (minloss>nbits && count==prevcount)
      return -1;
    prevcount=count;
  }
  if (debug_infolevel)
    *logptr(contextptr) << "Aberth mpfr: too many iterations " << N << " delta " << delta << "\n";
  return -1;  
}
#endif

dbl round(const dbl & z,const gen & pow2,int nbits){
  dbl res(z);
  round2(res,nbits);
  return res;
  gen n,d;
  fxnd(z,n,d);
  return iquo(n*pow2,d)/pow2;
}

bool aberth_z(const vdbl & P0,int nbits,int N,double eps,vdbl & R,int cluster_start,int cluster_afterend,vector<short int> & zi_done,GIAC_CONTEXT){
  int deg=P0.size()-1;
  bool doing_cluster=cluster_start>0 || cluster_afterend<deg;
  gen pow2=pow(2,nbits,contextptr);
  vdbl P(P0);
  dbl dr(0);
  dbl l(1);
  if (R.size()!=deg){
    dr=-find_shift(P,contextptr);
    P=shift(P,dr);
    //l=find_scale(P);
    //rescale(P,l);
    // init R if not already done, using single precision Aberth
    vfdbl fP,fR;
    if (convert(P,fP,contextptr)){
      vector<short int> zi_done(deg,false);
      aberth_singleprec(fP,N,1e-6,fR,0,deg,zi_done,3,contextptr);
      convert(fR,R);
    }
    else
      init_R(P,R);
  }
  P=*exact(P,contextptr)._VECTptr;
  R=*exact(R,contextptr)._VECTptr;
  vdbl Prev(P);
  reverse(Prev.begin(),Prev.end());
  vdbl newR(R);
  int ok=0; double delta;
  for (int k=0;k<N;++k){
    delta=0;
    if (doing_cluster){
      int K=cluster_start;
      for (;K<cluster_afterend;++K){
        if (!zi_done[K])
          break;
      }
      if (K==cluster_afterend)
        ok=1;
    }
    for (int i=0;i<deg;++i){
      if (zi_done[i]>=2 || (!ok && zi_done[i])){
        // do the last iteration if root is not already isolated
        newR[i]=R[i];
        continue;
      }
      dbl & zi=R[i];
      dbl d,d1;
      if (logabsdbl(zi)>0){
        dbl gamma(round(inv(zi),pow2,nbits));
        horner2(Prev,gamma,d,d1);
        if (!is_exactly_zero(d)){
          d=gamma*(dbl(deg)-gamma*d1/d);
          if (is_exactly_zero(d))
            return false;
          d=inv(d);
        }
      }
      else {
        horner2(P,zi,d,d1);
        if (is_exactly_zero(d1))
          return false;
        d=d/d1;
      }
      d=round(d,pow2,nbits);
      if (is_exactly_zero(d)){
        newR[i]=zi;
        zi_done[i]=1;
        continue;
      }
      dbl p(0); bool binv=true;
      for (int j=0;j<deg;++j){
	if (j==i) continue;
        dbl dz=zi-R[j];
        if (is_exactly_zero(dz)){
          if (doing_cluster)
            return false;
          binv=false;
          break;
        }
	p += round(inv(dz),pow2,nbits);
      }
      double dd=absdbl(d),pp=absdbl(p);
      double abszi=absdbl(zi);
      if (debug_infolevel>2)
        *logptr(contextptr) << "cluster? i=" << i << ", delta=" << dd << ", cluster_step=" << cluster_step << ", p=" << pp << ", d*p=" << dd*pp << " " << cluster_dp << "\n";      
      if (!binv ||
          (!ok && k>N/2 && !doing_cluster && dd<=cluster_step*abszi && dd*pp>=cluster_dp)){
        // cluster of roots, find all roots in this cluster
        int cend=i+1;
        dbl sumR=zi;
        for (int k=cend;k<deg;++k){
          if (absdbl(zi-R[k])<2*deg/pp){
            swap(R[cend],R[k]);
            swap(zi_done[cend],zi_done[k]);
            sumR+=R[cend];
            ++cend;
          }
        }
        if (cend-i>1){
          // cluster is from i to cend-1 included
          dbl z=sumR/dbl(cend-i); // center of gravity of cluster
          // shift P and reverse (roots become inverse of roots)
          vdbl Pcluster(shift(P,z,false)),Rcluster(deg),initR(deg);;
          reverse(Pcluster.begin(),Pcluster.end());
          // dbl D(dbl(cend-i)/d);
          int count=deg-1;
          for (int k=0;k<deg;++k){
            if (k>=i && k<cend){
              Rcluster[k]=initR[count];
              --count;
              //dbl a(0.0,0.7+(k-i)*2*M_PI/(cend-i));
              //Rcluster[k]=D*exp(a);
            }
            else
              Rcluster[k]=inv(R[k]-z);
          }
          vector<short int> old(zi_done);
          // recursive call of aberth
          if (debug_infolevel)
            *logptr(contextptr) << "aberth exact cluster=" << i << "," << cend << "\n";
          bool b=aberth_z(Pcluster,nbits,N,eps,Rcluster,i,cend,zi_done,contextptr);
          for (int k=0;k<deg;++k){
            if (!old[k])
              R[k]=l*(inv(Rcluster[k])+z)+dr;
          }
          if (b)
            continue;
          return false;
        }
      }
      double absdz=abszi==0?absdbl(d):absdbl(d)/abszi;
      if (binv)
        d=d/(dbl(1)-round(d*p,pow2,nbits));
      else {
        if (ok)
          return false;
        d=d/(1-exact(rand()/2.0/RAND_MAX,contextptr));
      } 
      delta += absdz;
      newR[i]=round(zi-d,pow2,nbits);
      if (absdz<eps/deg){
        if (debug_infolevel>1 && !zi_done[i])
          *logptr(contextptr) << "New root found " << newR[i] << "\n";
        zi_done[i]=1;
      }
    }
    newR.swap(R);
    int count=0;
    for (int k=0;k<deg;++k){
      if (zi_done[k])
        ++count;
    }
    if (debug_infolevel>0){
      *logptr(contextptr) << "Aberth exact bits="<< nbits << " iter="<<  k  << " found=" << count << " delta=" << delta << " cluster_search=" << cluster_start <<  ".." <<cluster_afterend << " time " << clock()*1e-6 << "\n";
      if (debug_infolevel>2){
        vdbl RR(R);
        accurate_evalf(RR,45);
        *logptr(contextptr) << "Current approx roots " << RR << "\n";
      }
    }
    if (doing_cluster){
      if (ok)
        return true;
      continue;
    }
    if (delta<=eps*(ok?deg:deg-count) || count==deg)
      ++ok;
    // else ok=0;
    if (ok==2){
      // rescale and translate
      for (size_t j=0;j<R.size();++j){
        R[j] = l*R[j] + dr;
      }
      return true;
    }
  }
  *logptr(contextptr) << "Aberth exact: too many iterations " << N << " delta " << delta << "\n";
  return false;  
}

bool Aberth(const vdbl & P0,vdbl & rayon,int N,double eps,vdbl & R,int isolate,bool do_exact,GIAC_CONTEXT){
  if (P0.size()==2){
    R.clear();
    R.push_back(-P0[1]/P0[0]);
    rayon.clear();
    rayon.push_back(-1);
    return true;
  }
  vdbl P(P0);
  int deg=P.size()-1;
  dbl dr=-find_shift(P,contextptr);
  P=shift(P,dr);
  bool realpoly=true;
  for (int i=0;realpoly && i<deg;++i){
    if (!is_exactly_zero(im(P[i],contextptr)))
      realpoly=false;
  }
  int bits=nbits_start; // 512;
  R.clear();
  int Nmax=N;
  rayon.resize(P.size()-1);
  vector<short int> zi_done(P.size()-1,0);
  int bit2=0,N2=0,eps2=0;
  double eps0=eps;
  while (bits<=giac::ABERTH_NBITSMAX){
    //zi_done=vector<short int>(deg,0);
#ifdef HAVE_LIBMPFR    
    int b=do_exact?aberth_z(P,bits,Nmax,eps,R,0,deg,zi_done,contextptr):aberth_mpfr(P,realpoly,bits,Nmax,eps,R,rayon,0,P.size()-1,zi_done,true,isolate,false/* secular*/,contextptr);
#else
    int b=aberth_z(P,bits,Nmax,eps,R,0,deg,zi_done,contextptr);
#endif
    bool bisol=b==2;
    if (!bisol && b!=-1)
      bisol=chk_isol(R,realpoly,isolate,rayon,eps0,zi_done,contextptr);
    if (b>0){
      // if isolate is true, chk_isol will set zi_done[i] to 2 (root i is isolated) or 0
      if (b==2 || bisol){
        for (size_t j=0;j<R.size();++j){
          R[j] += dr;
        }
        return true;
      }
      if (isolate){
        ++eps2;
        if (eps>1e-4)
          eps=1e-8;
        else if (eps>1e-40)
          eps=eps*eps;
        else if (eps<=1e-280)
          return false;
        else
          eps=eps*1e-40;
        *logptr(contextptr) << "Root isolation: setting epsilon to " << eps << "\n";
        if (eps2>bit2){
          ++bit2;
          bits *=2;
          clear_zi_done(zi_done);
        }
        continue;
      }
    }
    ++bit2;
    bits *= 2;
    clear_zi_done(zi_done);
    ++N2; Nmax*=1.2;
  }
  return false;
}

bool read_poly(const string & s,vdbl & P){
  try {
    gen g(s,contextptr);
    g=eval(g,1,contextptr);
    if (g.type!=_IDNT){
      if (g.type==_SYMB)
        g=_symb2poly(g,contextptr);
      if (g.type==_VECT){
        P=*g._VECTptr;
        return true;
      }
    }
  }
  catch (std::runtime_error & e){
  }
  P.clear();
  FILE * f=fopen(s.c_str(),"r");
  if (!f)
    return false;
  string S;
  while (1){
    char ch=fgetc(f);
    if (feof(f))
      break;
    S += ch;
  }
  fclose(f);
  gen g(S,contextptr);
  g=eval(g,1,contextptr);
  if (g.type==_SYMB)
    g=_symb2poly(g,contextptr);
  if (g.type==_VECT){
    P=*g._VECTptr;
  }
  return true;
}

vdbl p_coeff(const vdbl & R){
  vdbl P;
  P.reserve(R.size()+1);
  P.push_back(dbl(1));
  for (size_t i=0;i<R.size();++i){
    dbl z=R[i];
    // P*(x-z)
    P.push_back(-z*P.back());
    for (size_t j=P.size()-2;j>=1;--j){
      P[j] -= z*P[j-1];
    }
  }
  return P;
}

// max distance between 2 elements of R
double ecart(const vdbl & R){
  int n=R.size();
  double d=1e307;
  for (int i=0;i<n;++i){
    const dbl & Ri=R[i];
    for (int j=0;j<n;++j){
      if (j==i) continue;
      double dd=absdbl(Ri-R[j]);
      if (dd<d)
        d=dd;
    }
  }
  return d;
}

#if defined HAVE_LIBMPS && defined HAVE_LIBMPFR
// time mpsolve -au -o30 -Ga mand255.pol (Aberth)
// time mpsolve -as -o30 -Ga mand255.pol (secular)

bool gen2mpq(const gen & R,mpq_t & rq){
  if (R.type==_INT_){
    mpq_set_si(rq,R.val,1);
    return true;
  }
  if (R.type==_DOUBLE_){
    mpq_set_d(rq,R._DOUBLE_val);
    return true;
  }
  if (R.type==_ZINT){
    mpq_set_z(rq,*R._ZINTptr);
    return true;
  }
  if (R.type==_FRAC){
    gen num=R._FRACptr->num,den=R._FRACptr->den;
    num.uncoerce(); den.uncoerce();
    mpq_set_num(rq,*num._ZINTptr);
    mpq_set_den(rq,*den._ZINTptr);
    return true;
  }
  return false;
}

int Mps_solve(const vdbl & P,vdbl & R,vdbl & rayon,double eps,int isolate,bool secular,GIAC_CONTEXT){
  int n=P.size()-1,nmps=53; R.clear(); rayon.clear();
  mps_context *s;
  s = mps_context_new ();
  mps_context_select_algorithm(s, secular?MPS_ALGORITHM_SECULAR_GA:MPS_ALGORITHM_STANDARD_MPSOLVE);
  nmps=eps==0?giac::ABERTH_NBITSMAX:int(ceil(-log2(std::abs(eps))));;
  if (isolate){
    mps_context_set_output_goal (s, MPS_OUTPUT_GOAL_ISOLATE);
    *logptr(contextptr) << "MPS " << (secular?"secular":"Aberth") << " goal isolate, output bits=" << nmps << "\n";
  }
  else { 
    mps_context_set_output_goal (s, MPS_OUTPUT_GOAL_APPROXIMATE);
    *logptr(contextptr) << "MPS " << (secular?"secular":"Aberth") << " goal approximate, output bits=" << nmps << "\n";
  }
  mps_context_set_output_prec (s, nmps);
  //mps_context_set_input_prec(s,0);
  int I;
  /*
  for (I=0;I<=n;++I){
    if (!is_cinteger(P[I]))
      break;
  }
  */
  mps_monomial_poly *p=0;
  if (0 && I<=n){
    string S=symb_horner(P,vx_var).print(contextptr);
    p = MPS_MONOMIAL_POLY (mps_parse_inline_poly_from_string (s,S.c_str()));
  }
  else {
    p = mps_monomial_poly_new (s, n);
    mpq_t rq,iq;
    mpq_init(rq); mpq_init(iq);
    for (int i=0;i<=n;++i){
      dbl R,I;
      reim( P[n-i],R,I,contextptr);
      if (R.type==_DOUBLE_ || I.type==_DOUBLE_ || R.type==_REAL || I.type==_REAL){
        R=evalf_double(R,1,contextptr);
        I=evalf_double(I,1,contextptr);
        mps_monomial_poly_set_coefficient_d (s, p, i,R._DOUBLE_val,I._DOUBLE_val);
        continue;
      }
      if (!gen2mpq(R,rq))
        return -1;
      if (!gen2mpq(I,iq))
        return -1;
      mps_monomial_poly_set_coefficient_q (s, p, i,rq,iq);
    }
    mpq_clear(rq); mpq_clear(iq);
    /* 
       
       mps_monomial_poly_set_coefficient_q (s, p, 0, m_one, zero); 
       mps_monomial_poly_set_coefficient_q (s, p, n, one, zero);  
       
       for (int i=0;i<=n;++i){
       dbl R,I;
       reim( evalf_double(P[n-i],1,contextptr),R,I,contextptr);
       mps_monomial_poly_set_coefficient_d (s, p, i,R._DOUBLE_val, I._DOUBLE_val); 
       }
    */
  }
  /* Set the input polynomial */
  mps_context_set_input_poly (s, MPS_POLYNOMIAL (p));
  
  /* Actually solve the polynomial */
  double t1=CLOCK()*1e-6;
  mps_mpsolve (s);
  double t2=CLOCK()*1e-6;
  *logptr(contextptr) << "MPS solve time " << t2-t1 << "\n";
  
  R.clear();
  rdpe_t * drad = rdpe_valloc (n); // disk radius
  mpc_t * mroot = mpc_valloc (n);
  mpc_vinit2 (mroot, n, nmps);   
  mps_context_get_roots_m (s, &mroot, &drad);
  mpfr_t tmp; mpfr_init2(tmp,64);
  for (int i=0;i<n;++i){
    mpfr_t mr,mi; dbl gr,gi;
    double d1=rdpe_log(drad[i]);
    rayon.push_back(exp(d1));
    mpfr_set_f(tmp,mpc_Re(mroot[i]),MPFR_RNDN);
    gr=gen(real_object(tmp));
    if (!is_exactly_zero(gr)){
      double d2=logabsdbl(gr);
      double d=d2-d1;
      if (d<0)
        gr=0;
      else {
        int prec=d/M_LN2; // mpf_get_prec(mpc_Re(mroot[i]));
        mpfr_init2(mr,prec); 
        mpfr_set_f(mr,mpc_Re(mroot[i]),MPFR_RNDN);
        gr=gen(real_object(mr));
        mpfr_clear(mr);
      }
    }
    mpfr_set_f(tmp,mpc_Im(mroot[i]),MPFR_RNDN);
    gi=gen(real_object(tmp));
    if (!is_exactly_zero(gi)){
      // cout << i << " " << gi << "\n";
      double d2=logabsdbl(gi);
      double d=d2-d1;
      if (d<0)
        gi=0;
      else {
        int prec=d/M_LN2; // mpf_get_prec(mpc_Im(mroot[i]));
        mpfr_init2(mi,prec);
        mpfr_set_f(mi,mpc_Im(mroot[i]),MPFR_RNDN);
        gi=gen(real_object(mi));
        mpfr_clear(mi);
      }
    }
    R.push_back(gen(gr,gi));
  }
  mpfr_clear(tmp);
  mpc_vclear (mroot, n);
  free (mroot);
  free (drad);
  // mps_monomial_poly_free(s,p); // expects a mps_polynomial * ??
  mps_context_free(s);
  return 0;
}
#endif

// Random check with a.out eps n, eps>0 (Aberth) and n<0 (random coeff) or n>0 (random roots)
int main(int argc,char ** argv){
  //srand(time(NULL));
  int maxdig=80;
  if (getenv("GIAC_MAXDIGITS"))
    maxdig=atoi(getenv("GIAC_MAXDIGITS"));
  int maxd=maxdig*std::log(10);
  int do_exact=0,do_isolate=3;
  bool secular=true;
  if (getenv("GIAC_EXACT"))
    do_exact=atoi(getenv("GIAC_EXACT"));
  if (getenv("GIAC_SECULAR"))
    secular=atoi(getenv("GIAC_SECULAR"));
  if (getenv("GIAC_ISOLATE"))
    do_isolate=atoi(getenv("GIAC_ISOLATE"));
  if (getenv("GIAC_DEBUG")){
    debug_infolevel=atoi(getenv("GIAC_DEBUG"));
  }
  if (getenv("GIAC_NMAX")){
    Nmax=atoi(getenv("GIAC_NMAX"));
  }
  if (getenv("GIAC_N1")){
    giac::ABERTH_NBITSMAX=atoi(getenv("GIAC_N1"));
  }
  if (getenv("GIAC_N0")){
    nbits_start=atoi(getenv("GIAC_N0"));
  }
  if (getenv("GIAC_STEP")){
    cluster_step=atof(getenv("GIAC_STEP"));
  }
  if (getenv("GIAC_DP")){
    cluster_dp=atof(getenv("GIAC_DP"));
  }
  cout << setprecision(16)
       << "Configuration debug GIAC_DEBUG=" << debug_infolevel
       << " isolate GIAC_ISOLATE=" << do_isolate
       << " start bits precision GIAC_N0=" << nbits_start 
       << " max bits precision GIAC_N1=" << giac::ABERTH_NBITSMAX 
       << " initial max iterations GIAC_NMAX=" << Nmax
       << " cluster max step GIAC_STEP=" << cluster_step
       << " cluster max dp GIAC_DP=" << cluster_dp 
       << " max number of digits in output GIAC_MAXDIGITS=" << maxdig
       << "\n";
  if (argc<3){
    cout << "Syntax " << argv[0] << " eps [filename|x-polynomial]\n";
    cout << "GIAC_N GIAC_NMAX GIAC_DEBUG GIAC_STEP GIAC_DP\n";
    return 1;
  }
  double eps=1;
  vdbl P,R;
  eps=atof(argv[1]);
  bool runmps=false;
  if (eps<=0){
    eps=-eps;
    runmps=true;
  }
  if (eps>1e-4){
    cout << "Syntax " << argv[0] << " eps x-polynomial (replace with N or with root1 ... rootn to check)\n";
    return 1;
  }
  int dig=-log10(eps);
  if (argc==3){
    int n=0;
    if (argv[1][0]=='-' || (argv[1][0]>'0' && argv[1][0]<='9') )
      n=atoi(argv[2]);
    if (n==0){
      if (!read_poly(argv[2],P))
	return 1;
    }
    else {
      int N=n<0?-n:n;
      dbl M(1/(RAND_MAX+1.0));
      if (n<0)
        R.push_back(dbl(1.0));
      for (int i=0;i<N;++i){
	dbl z(rand(),rand());
	z=M*z;
	R.push_back(z);
      }
      if (n<0)
	P=R;
      else {
	sort(R.begin(),R.end(),dbl_less);
	cout << "Random roots ecart " << ecart(R) << "\nRoots: " << R << "\n";
	accurate_evalf(R,nbits_start);
	P=p_coeff(R);
      }
    }
  }
  else { // check: polynom given by (real) roots
    for (int i=2;i<argc;++i){
      R.push_back(atof(argv[i]));
    }
    accurate_evalf(R,nbits_start);
    P=p_coeff(R);
  }
  if (has_num_coeff(P)){
    accurate_evalf(P,nbits_start);
    do_exact=false;
  }
  else {
    // gen l; lcmdeno(P,l,contextptr);
  }
  vdbl PP(P);
  accurate_evalf(PP,45);
  if (P.size()<=32 || debug_infolevel>1)
    cout << "Polynom "  << PP << "\n";
  R.clear();
  vdbl RR;
  int deg=P.size()-1;
  vdbl rayon(deg);
#ifdef HAVE_LIBMPS
  if (runmps){
    Mps_solve(P,R,rayon,eps,do_isolate==2,secular,contextptr);
  } else
#endif // MPS
    {
      double t1=CLOCK()*1e-6;
      if (!Aberth(P,rayon,Nmax,eps,R,do_isolate,do_exact,contextptr)){
        cout << "Unable to find roots\n";
        return 2;
      }
      double t2=CLOCK()*1e-6;
      *logptr(contextptr) << "Isolation time " << t2-t1 << "\n";
    }
  ofstream of(runmps?"roots0":"roots1");
  for (int i=0;i<deg;++i){
    dbl Ri=R[i],Rir,Rii;
    reim(Ri,Rir,Rii,contextptr);
    double r=logabsdbl(rayon[i]);
    int prec=logabsdbl(Rir)-r;
    if (prec>maxd) prec=maxd;
    if (prec<0)
      Rir=0;
    else
      Rir=accurate_evalf(Rir,ceil(prec/M_LN2)+4);
    prec=logabsdbl(Rii)-r;
    if (prec>maxd) prec=maxd;
    if (prec<0)
      Rii=0;
    else
      Rii=accurate_evalf(Rii,ceil(prec/M_LN2)+4);
    RR.push_back(makevecteur(gen(Rir,Rii),rayon[i]));
  }
  sort(RR.begin(),RR.end(),dblrayon_less); 
  for (int i=0;i<deg;++i){
    double r=logabsdbl(RR[i][1]);
    double d=r/log(10.0);
    int e=floor(d);
    d=ceil(pow(10,d-e));
    of << RR[i][0] << " " << d ;
    if (d!=0)
      of << "e" << e;
    of << "\n";
  }
  if (P.size()<=16 || debug_infolevel>1)
    cout << "Result roots " << RR << "\n";
  vdbl PR(p_coeff(R));
  if (debug_infolevel>=2)
    cout << "Coeff precisions ";
  double prec=0;
  for (size_t i=1;i<PR.size();++i){
    double a=absdbl(P[0]*PR[i]-P[i]),b=absdbl(P[i]);
    double c=(b==0?a:a/b);
    if (c>prec)
      prec=c;
    if (debug_infolevel>=2)
      cout << c << " " ;
  }
  cout << "\nMax. coeff precision : " << (prec==0?1e-310:prec) << "\n";
}
